@@ -1,5 +1,10 @@
 extern crate cmake;
 
+#[cfg(feature = "docs-rs")]
+fn main() {
+}
+
+#[cfg(not(feature = "docs-rs"))]
 fn main() {
     let dst = cmake::Config::new("libvulkanprofiles").build();
 
